@@ -11,7 +11,9 @@ package org.eclipse.egit.core.internal.indexdiff;
 import java.io.IOException;
 import java.text.MessageFormat;
 import java.util.Collection;
+import java.util.Collections;
 import java.util.HashSet;
+import java.util.List;
 import java.util.Set;
 import java.util.concurrent.locks.ReentrantLock;
 
@@ -19,7 +21,7 @@ import org.eclipse.core.resources.IProject;
 import org.eclipse.core.resources.IResource;
 import org.eclipse.core.resources.IResourceChangeEvent;
 import org.eclipse.core.resources.IResourceChangeListener;
-import org.eclipse.core.resources.IWorkspaceRunnable;
+import org.eclipse.core.resources.IWorkspaceRoot;
 import org.eclipse.core.resources.ResourcesPlugin;
 import org.eclipse.core.runtime.CoreException;
 import org.eclipse.core.runtime.IProgressMonitor;
@@ -150,6 +152,16 @@ public class IndexDiffCacheEntry {
 		scheduleReloadJob("Refresh called"); //$NON-NLS-1$
 	}
 
+	/**
+	 * Trigger a new index diff calculation manually for the passed files.
+	 *
+	 * @param filesToRefresh (repository-relative paths)
+	 */
+	public void refreshFiles(final Collection<String> filesToRefresh) {
+		List<IResource> resources = Collections.emptyList();
+		scheduleUpdateJob(filesToRefresh, resources);
+	}
+
 	/**
 	 * The method returns the current index diff or null. Null is returned if
 	 * the first index diff calculation has not completed yet.
@@ -232,17 +244,13 @@ public class IndexDiffCacheEntry {
 		// branch switch).
 		// The index diff calculation jobs do not lock the workspace
 		// during execution to avoid blocking the workspace.
+		IWorkspaceRoot root = ResourcesPlugin.getWorkspace().getRoot();
 		try {
-			ResourcesPlugin.getWorkspace().run(new IWorkspaceRunnable() {
-
-				public void run(IProgressMonitor innerMonitor) throws CoreException {
-					// empty
-				}
-			}, monitor);
+			Job.getJobManager().beginRule(root, monitor);
 		} catch (OperationCanceledException e) {
 			return;
-		} catch (CoreException e) {
-			throw new RuntimeException(e);
+		} finally {
+			Job.getJobManager().endRule(root);
 		}
 	}
 
