@@ -22,6 +22,8 @@ import org.eclipse.core.runtime.SubMonitor;
 import org.eclipse.egit.core.Activator;
 import org.eclipse.egit.core.synchronize.dto.GitSynchronizeData;
 import org.eclipse.egit.core.synchronize.dto.GitSynchronizeDataSet;
+import org.eclipse.jgit.dircache.DirCache;
+import org.eclipse.jgit.dircache.DirCacheIterator;
 import org.eclipse.jgit.lib.AbbreviatedObjectId;
 import org.eclipse.jgit.lib.ObjectId;
 import org.eclipse.jgit.lib.Repository;
@@ -109,8 +111,10 @@ class GitSyncCache {
 
 		try {
 			// setup local tree
+			FileTreeIterator fti = null;
 			if (gsd.shouldIncludeLocal()) {
-				tw.addTree(new FileTreeIterator(repo));
+				fti = new FileTreeIterator(repo);
+				tw.addTree(fti);
 				if (filter != null)
 					tw.setFilter(AndTreeFilter.create(filter,
 							new NotIgnoredFilter(0)));
@@ -133,6 +137,12 @@ class GitSyncCache {
 			else
 				tw.addTree(new EmptyTreeIterator());
 
+			DirCacheIterator dci = null;
+			if (fti != null) {
+				dci = new DirCacheIterator(DirCache.read(repo));
+				tw.addTree(dci);
+				fti.setDirCacheIterator(tw, 3);
+			}
 			List<ThreeWayDiffEntry> diffEntrys = ThreeWayDiffEntry.scan(tw);
 			tw.release();
 
