@@ -51,6 +51,7 @@ import org.eclipse.jgit.lib.Ref;
 import org.eclipse.jgit.lib.Repository;
 import org.eclipse.jgit.revwalk.RevCommit;
 import org.eclipse.jgit.revwalk.RevWalk;
+import org.eclipse.jgit.revwalk.RevWalkUtils;
 import org.eclipse.swt.SWT;
 import org.eclipse.swt.custom.CLabel;
 import org.eclipse.swt.events.DisposeEvent;
@@ -482,25 +483,17 @@ public class CommitEditorPage extends FormPage implements ISchedulingRule {
 		Repository repository = getCommit().getRepository();
 		RevCommit commit = getCommit().getRevCommit();
 		RevWalk revWalk = new RevWalk(repository);
-		List<Ref> result = new ArrayList<Ref>();
 		try {
 			Map<String, Ref> refsMap = new HashMap<String, Ref>();
 			refsMap.putAll(repository.getRefDatabase().getRefs(
 					Constants.R_HEADS));
 			refsMap.putAll(repository.getRefDatabase().getRefs(
 					Constants.R_REMOTES));
-			for (Ref ref : refsMap.values()) {
-				if (ref.isSymbolic())
-					continue;
-				RevCommit headCommit = revWalk.parseCommit(ref.getObjectId());
-				RevCommit base = revWalk.parseCommit(commit);
-				if (revWalk.isMergedInto(base, headCommit))
-					result.add(ref);
-			}
-		} catch (IOException ignored) {
-			// Ignored
+			return RevWalkUtils.findBranchesReachableFrom(commit, revWalk, refsMap.values());
+		} catch (IOException e) {
+			Activator.handleError(e.getMessage(), e, false);
+			return Collections.emptyList();
 		}
-		return result;
 	}
 
 	private void loadSections() {
