@@ -223,6 +223,17 @@ public class CommitHelper {
 		return previousCommit != null && repository.getRepositoryState().canAmend();
 	}
 
+	/**
+	 * @param repository
+	 *            to check
+	 * @return true if an empty commit without files is allowed in the
+	 *         current state
+	 */
+	public static boolean isCommitWithoutFilesAllowed(Repository repository) {
+		RepositoryState state = repository.getRepositoryState();
+		return state == RepositoryState.MERGING_RESOLVED;
+	}
+
 	/**
 	 * @param repository
 	 * @return info related to the HEAD commit
