@@ -51,7 +51,6 @@ import org.eclipse.swt.graphics.Color;
 import org.eclipse.swt.graphics.Font;
 import org.eclipse.swt.graphics.ImageData;
 import org.eclipse.swt.widgets.Display;
-import org.eclipse.team.core.Team;
 import org.eclipse.team.internal.ui.Utils;
 import org.eclipse.team.ui.ISharedImages;
 import org.eclipse.team.ui.TeamImages;
@@ -209,11 +208,6 @@ public class GitLightweightDecorator extends LabelProvider implements
 		if (mapping == null)
 			return null;
 
-		// Don't decorate ignored resources (e.g. bin folder content)
-		if (resource.getType() != IResource.PROJECT
-				&& Team.isIgnoredHint(resource))
-			return null;
-
 		// Cannot decorate linked resources
 		if (mapping.getRepoRelativePath(resource) == null)
 			return null;
