@@ -5,6 +5,8 @@
  * Copyright (C) 2008, Robin Rosenberg <robin.rosenberg@dewire.com>
  * Copyright (C) 2007, Shawn O. Pearce <spearce@spearce.org>
  * Copyright (C) 2011, Mathias Kinzler <mathias.kinzler@sap.com>
+ * Copyright (C) 2012, Daniel Megert <daniel_megert@ch.ibm.com>
+ * Copyright (C) 2012, Robin Stocker <robin@nibor.org>
  *
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
@@ -25,18 +27,25 @@ import java.util.Iterator;
 import java.util.Set;
 
 import org.eclipse.core.resources.IFile;
+import org.eclipse.core.resources.IMarker;
+import org.eclipse.core.resources.IResource;
 import org.eclipse.core.resources.ResourcesPlugin;
 import org.eclipse.core.runtime.Assert;
+import org.eclipse.core.runtime.CoreException;
 import org.eclipse.core.runtime.Path;
 import org.eclipse.egit.core.Activator;
 import org.eclipse.egit.core.AdaptableFileTreeIterator;
+import org.eclipse.egit.core.internal.util.ResourceUtil;
 import org.eclipse.egit.ui.UIIcons;
 import org.eclipse.egit.ui.UIPreferences;
 import org.eclipse.egit.ui.UIText;
 import org.eclipse.egit.ui.UIUtils;
 import org.eclipse.egit.ui.internal.CompareUtils;
+import org.eclipse.egit.ui.internal.commit.CommitHelper;
 import org.eclipse.egit.ui.internal.commit.CommitMessageHistory;
 import org.eclipse.egit.ui.internal.commit.CommitProposalProcessor;
+import org.eclipse.egit.ui.internal.decorators.IProblemDecoratable;
+import org.eclipse.egit.ui.internal.decorators.ProblemLabelDecorator;
 import org.eclipse.egit.ui.internal.dialogs.CommitItem.Status;
 import org.eclipse.egit.ui.internal.dialogs.CommitMessageComponent.CommitStatus;
 import org.eclipse.jface.dialogs.IDialogConstants;
@@ -55,14 +64,19 @@ import org.eclipse.jface.resource.ResourceManager;
 import org.eclipse.jface.text.DocumentEvent;
 import org.eclipse.jface.text.IDocumentListener;
 import org.eclipse.jface.viewers.ArrayContentProvider;
+import org.eclipse.jface.viewers.BaseLabelProvider;
+import org.eclipse.jface.viewers.CellLabelProvider;
 import org.eclipse.jface.viewers.CheckStateChangedEvent;
 import org.eclipse.jface.viewers.CheckboxTableViewer;
 import org.eclipse.jface.viewers.ColumnLabelProvider;
 import org.eclipse.jface.viewers.ColumnViewerToolTipSupport;
+import org.eclipse.jface.viewers.DecoratingStyledCellLabelProvider;
 import org.eclipse.jface.viewers.DecorationOverlayIcon;
+import org.eclipse.jface.viewers.DelegatingStyledCellLabelProvider.IStyledLabelProvider;
 import org.eclipse.jface.viewers.ICheckStateListener;
 import org.eclipse.jface.viewers.IDecoration;
 import org.eclipse.jface.viewers.IStructuredSelection;
+import org.eclipse.jface.viewers.StyledString;
 import org.eclipse.jface.viewers.TableViewerColumn;
 import org.eclipse.jface.viewers.Viewer;
 import org.eclipse.jface.viewers.ViewerComparator;
@@ -78,8 +92,6 @@ import org.eclipse.jgit.treewalk.filter.PathFilterGroup;
 import org.eclipse.swt.SWT;
 import org.eclipse.swt.events.DisposeEvent;
 import org.eclipse.swt.events.DisposeListener;
-import org.eclipse.swt.events.KeyAdapter;
-import org.eclipse.swt.events.KeyEvent;
 import org.eclipse.swt.events.ModifyEvent;
 import org.eclipse.swt.events.ModifyListener;
 import org.eclipse.swt.events.SelectionAdapter;
@@ -119,7 +131,8 @@ public class CommitDialog extends TitleAreaDialog {
 		return org.eclipse.egit.ui.Activator.getDefault().getPreferenceStore();
 	}
 
-	static class CommitStatusLabelProvider extends ColumnLabelProvider {
+	static class CommitStatusLabelProvider extends BaseLabelProvider implements
+			IStyledLabelProvider {
 
 		private Image DEFAULT = PlatformUI.getWorkbench().getSharedImages()
 				.getImage(ISharedImages.IMG_OBJ_FILE);
@@ -149,8 +162,8 @@ public class CommitDialog extends TitleAreaDialog {
 			return (Image) this.resourceManager.get(decorated);
 		}
 
-		public String getText(Object obj) {
-			return ""; //$NON-NLS-1$
+		public StyledString getStyledText(Object element) {
+			return new StyledString();
 		}
 
 		public Image getImage(Object element) {
@@ -176,16 +189,12 @@ public class CommitDialog extends TitleAreaDialog {
 					decorator) : getEditorImage(item);
 		}
 
-		public String getToolTipText(Object element) {
-			return ((CommitItem) element).status.getText();
-		}
-
+		@Override
 		public void dispose() {
 			SUBMODULE.dispose();
 			resourceManager.dispose();
 			super.dispose();
 		}
-
 	}
 
 	static class CommitPathLabelProvider extends ColumnLabelProvider {
@@ -345,6 +354,8 @@ public class CommitDialog extends TitleAreaDialog {
 
 	private Repository repository;
 
+	private boolean pushEnabled = false;
+
 	/**
 	 * @param parentShell
 	 */
@@ -411,6 +422,7 @@ public class CommitDialog extends TitleAreaDialog {
 			item.status = getFileStatus(path, indexDiff);
 			item.submodule = FileMode.GITLINK == indexDiff.getIndexMode(path);
 			item.path = path;
+			item.problemSeverity = getProblemSeverity(repository, path);
 			items.add(item);
 		}
 
@@ -497,6 +509,14 @@ public class CommitDialog extends TitleAreaDialog {
 		return createChangeId;
 	}
 
+	/**
+	 * Returns whether we are pushing after the commit
+	 * @return pushing
+	 */
+	public boolean isPushEnabled() {
+		return pushEnabled;
+	}
+
 	@Override
 	protected void createButtonsForButtonBar(Composite parent) {
 		toolkit.adapt(parent, false, false);
@@ -641,20 +661,6 @@ public class CommitDialog extends TitleAreaDialog {
 		UIUtils.addBulbDecorator(commitText.getTextWidget(),
 				UIText.CommitDialog_ContentAssist);
 
-		// allow to commit with ctrl-enter
-		commitText.getTextWidget().addKeyListener(new KeyAdapter() {
-			public void keyPressed(KeyEvent event) {
-				if (event.keyCode == SWT.CR
-						&& (event.stateMask & SWT.CONTROL) > 0) {
-					okPressed();
-				} else if (event.keyCode == SWT.TAB
-						&& (event.stateMask & SWT.SHIFT) == 0) {
-					event.doit = false;
-					commitText.traverse(SWT.TRAVERSE_TAB_NEXT);
-				}
-			}
-		});
-
 		Composite personArea = toolkit.createComposite(container);
 		toolkit.paintBordersFor(personArea);
 		GridLayoutFactory.swtDefaults().numColumns(2).applyTo(personArea);
@@ -783,7 +789,7 @@ public class CommitDialog extends TitleAreaDialog {
 
 		filesViewer = new CheckboxTableViewer(resourcesTable);
 		new TableViewerColumn(filesViewer, statCol)
-				.setLabelProvider(new CommitStatusLabelProvider());
+				.setLabelProvider(createStatusLabelProvider());
 		new TableViewerColumn(filesViewer, resourceCol)
 				.setLabelProvider(new CommitPathLabelProvider());
 		ColumnViewerToolTipSupport.enableFor(filesViewer);
@@ -884,6 +890,32 @@ public class CommitDialog extends TitleAreaDialog {
 			}
 		}
 
+		Section pushSection = toolkit.createSection(container,
+				ExpandableComposite.TITLE_BAR
+						| ExpandableComposite.CLIENT_INDENT);
+		GridDataFactory.fillDefaults().grab(true, true).applyTo(pushSection);
+		Composite pushArea = toolkit.createComposite(pushSection);
+		pushSection.setClient(pushArea);
+		toolkit.paintBordersFor(pushArea);
+		GridLayoutFactory.fillDefaults().extendedMargins(2, 2, 2, 2)
+				.applyTo(pushArea);
+		pushSection.setText(UIText.CommitDialog_PushSectionTitle);
+		final Button pushCheckbox = toolkit.createButton(pushArea,
+				UIText.CommitDialog_PushUpstream, SWT.CHECK);
+		pushCheckbox.setSelection(getPreferenceStore().getBoolean(
+					UIPreferences.COMMIT_DIALOG_PUSH_UPSTREAM));
+		pushEnabled = getPreferenceStore().getBoolean(
+				UIPreferences.COMMIT_DIALOG_PUSH_UPSTREAM);
+		pushCheckbox.addSelectionListener(new SelectionAdapter() {
+
+			@Override
+			public void widgetSelected(SelectionEvent e) {
+				pushEnabled = pushCheckbox.getSelection();
+				getPreferenceStore().setValue(
+						UIPreferences.COMMIT_DIALOG_PUSH_UPSTREAM, pushEnabled);
+			}
+		});
+
 		applyDialogFont(container);
 		statCol.pack();
 		resourceCol.pack();
@@ -924,6 +956,17 @@ public class CommitDialog extends TitleAreaDialog {
 		return container;
 	}
 
+	private static CellLabelProvider createStatusLabelProvider() {
+		CommitStatusLabelProvider baseProvider = new CommitStatusLabelProvider();
+		ProblemLabelDecorator decorator = new ProblemLabelDecorator(null);
+		return new DecoratingStyledCellLabelProvider(baseProvider, decorator, null) {
+			@Override
+			public String getToolTipText(Object element) {
+				return ((CommitItem) element).status.getText();
+			}
+		};
+	}
+
 	private void updateMessage() {
 		String message = null;
 		int type = IMessageProvider.NONE;
@@ -932,8 +975,7 @@ public class CommitDialog extends TitleAreaDialog {
 		if (commitMsg == null || commitMsg.trim().length() == 0) {
 			message = UIText.CommitDialog_Message;
 			type = IMessageProvider.INFORMATION;
-		} else if (filesViewer.getCheckedElements().length == 0
-				&& !amendingItem.getSelection()) {
+		} else if (!isCommitWithoutFilesAllowed()) {
 			message = UIText.CommitDialog_MessageNoFilesSelected;
 			type = IMessageProvider.INFORMATION;
 		} else {
@@ -947,6 +989,16 @@ public class CommitDialog extends TitleAreaDialog {
 				|| type == IMessageProvider.NONE);
 	}
 
+	private boolean isCommitWithoutFilesAllowed() {
+		if (filesViewer.getCheckedElements().length > 0)
+			return true;
+
+		if (amendingItem.getSelection())
+			return true;
+
+		return CommitHelper.isCommitWithoutFilesAllowed(repository);
+	}
+
 	private Collection<String> getFileList() {
 		Collection<String> result = new ArrayList<String>();
 		for (CommitItem item : items) {
@@ -1059,8 +1111,26 @@ public class CommitDialog extends TitleAreaDialog {
 		return Status.UNKNOWN;
 	}
 
+	private static int getProblemSeverity(Repository repository, String path) {
+		IFile file = ResourceUtil.getFileForLocation(repository, path);
+		if (file != null) {
+			try {
+				int severity = file.findMaxProblemSeverity(IMarker.PROBLEM, true, IResource.DEPTH_ONE);
+				return severity;
+			} catch (CoreException e) {
+				// Fall back to below
+			}
+		}
+		return IProblemDecoratable.SEVERITY_NONE;
+	}
+
 	@Override
 	protected void okPressed() {
+		if (!isCommitWithoutFilesAllowed()) {
+			MessageDialog.openWarning(getShell(), UIText.CommitDialog_ErrorNoItemsSelected, UIText.CommitDialog_ErrorNoItemsSelectedToBeCommitted);
+			return;
+		}
+
 		if (!commitMessageComponent.checkCommitInfo())
 			return;
 
@@ -1075,11 +1145,6 @@ public class CommitDialog extends TitleAreaDialog {
 		committer = commitMessageComponent.getCommitter();
 		createChangeId = changeIdItem.getSelection();
 
-		if (selectedFiles.isEmpty() && !amending) {
-			MessageDialog.openWarning(getShell(), UIText.CommitDialog_ErrorNoItemsSelected, UIText.CommitDialog_ErrorNoItemsSelectedToBeCommitted);
-			return;
-		}
-
 		IDialogSettings settings = org.eclipse.egit.ui.Activator
 			.getDefault().getDialogSettings();
 		settings.put(SHOW_UNTRACKED_PREF, showUntracked);
@@ -1103,13 +1168,20 @@ public class CommitDialog extends TitleAreaDialog {
 	}
 }
 
-class CommitItem {
+class CommitItem implements IProblemDecoratable {
+
 	Status status;
 
 	String path;
 
 	boolean submodule;
 
+	int problemSeverity;
+
+	public int getProblemSeverity() {
+		return problemSeverity;
+	}
+
 	/** The ordinal of this {@link Enum} is used to provide the "native" sorting of the list */
 	public static enum Status {
 		/** */
@@ -1180,7 +1252,6 @@ class CommitViewerComparator extends ViewerComparator {
 		super(comparator);
 	}
 
-	@SuppressWarnings("unchecked")
 	@Override
 	public int compare(Viewer viewer, Object e1, Object e2) {
 		return getComparator().compare(e1, e2);
