@@ -6,6 +6,7 @@
  * Copyright (C) 2011-2012, Mathias Kinzler <mathias.kinzler@sap.com>
  * Copyright (C) 2011-2012, Matthias Sohn <matthias.sohn@sap.com>
  * Copyright (C) 2012, Robin Stocker <robin@nibor.org>
+ * Copyright (C) 2012, Daniel Megert <daniel_megert@ch.ibm.com>
  *
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
@@ -44,6 +45,7 @@ import org.eclipse.jface.action.Action;
 import org.eclipse.jface.action.IAction;
 import org.eclipse.jface.action.MenuManager;
 import org.eclipse.jface.action.Separator;
+import org.eclipse.jface.resource.ResourceManager;
 import org.eclipse.jface.text.AbstractHoverInformationControlManager;
 import org.eclipse.jface.text.AbstractReusableInformationControlCreator;
 import org.eclipse.jface.text.DefaultInformationControl;
@@ -158,7 +160,8 @@ class CommitGraphTable {
 
 	private boolean trace = GitTraceLocation.HISTORYVIEW.isActive();
 
-	CommitGraphTable(Composite parent, final TableLoader loader) {
+	CommitGraphTable(Composite parent, final TableLoader loader,
+			final ResourceManager resources) {
 		nFont = UIUtils.getFont(UIPreferences.THEME_CommitGraphNormalFont);
 		hFont = highlightFont();
 		tableLoader = loader;
@@ -202,7 +205,7 @@ class CommitGraphTable {
 
 		table.setLabelProvider(graphLabelProvider);
 		table.setContentProvider(new GraphContentProvider());
-		renderer = new SWTPlotRenderer(rawTable.getDisplay());
+		renderer = new SWTPlotRenderer(rawTable.getDisplay(), resources);
 
 		clipboard = new Clipboard(rawTable.getDisplay());
 		rawTable.addDisposeListener(new DisposeListener() {
@@ -235,8 +238,6 @@ class CommitGraphTable {
 			public void widgetDisposed(DisposeEvent e) {
 				if (allCommits != null)
 					allCommits.dispose();
-				if (renderer != null)
-					renderer.dispose();
 				hoverManager.dispose();
 			}
 		});
@@ -247,8 +248,9 @@ class CommitGraphTable {
 	}
 
 	CommitGraphTable(final Composite parent, final IPageSite site,
-			final MenuManager menuMgr, final TableLoader loader) {
-		this(parent, loader);
+			final MenuManager menuMgr, final TableLoader loader,
+			final ResourceManager resources) {
+		this(parent, loader, resources);
 
 		final IAction selectAll = createStandardAction(ActionFactory.SELECT_ALL);
 		getControl().addFocusListener(new FocusListener() {
@@ -351,7 +353,6 @@ class CommitGraphTable {
 		return !table.getSelection().isEmpty();
 	}
 
-	@SuppressWarnings("unchecked")
 	private void doCopy() {
 		final ISelection s = table.getSelection();
 		if (s.isEmpty() || !(s instanceof IStructuredSelection))
@@ -746,6 +747,12 @@ class CommitGraphTable {
 							UIText.CommitFileDiffViewer_ShowAnnotationsMenuLabel));
 			}
 
+			if (selectionSize > 0) {
+				popupMgr.add(getCommandContributionItem(
+						HistoryViewCommands.OPEN_IN_COMMIT_VIEWER,
+						UIText.CommitGraphTable_OpenCommitLabel));
+			}
+
 			if (selectionSize == 1) {
 				popupMgr.add(new Separator());
 				popupMgr.add(getCommandContributionItem(
@@ -844,9 +851,6 @@ class CommitGraphTable {
 			// copy and such after additions
 			popupMgr.add(new Separator(IWorkbenchActionConstants.MB_ADDITIONS));
 			popupMgr.add(copyAction);
-			popupMgr.add(getCommandContributionItem(
-					HistoryViewCommands.OPEN_IN_COMMIT_VIEWER,
-					UIText.CommitGraphTable_OpenCommitLabel));
 			popupMgr.add(new Separator());
 		}
 
