@@ -1,5 +1,6 @@
 /*******************************************************************************
  * Copyright (C) 2010, Dariusz Luksza <dariusz@luksza.org>
+ * Copyright (C) 2012, Daniel Megert <daniel_megert@ch.ibm.com>
  *
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
@@ -40,14 +41,6 @@ public class GitChangeSetLabelProvider extends SynchronizationLabelProvider impl
 	/** */
 	public static final String BINDING_CHANGESET_DATE = "{date}"; //$NON-NLS-1$
 
-	/** */
-	public static final String DEFAULT_CHANGESET_FORMAT = String.format("[%s] (%s) %s", //$NON-NLS-1$
-			BINDING_CHANGESET_AUTHOR,
-			BINDING_CHANGESET_DATE,
-			BINDING_CHANGESET_SHORT_MESSAGE);
-
-	/** */
-	public static final String DEFAULT_DATE_FORMAT = "yyyy-MM-dd HH:mm:ss";  //$NON-NLS-1$
 
 	private IPreferenceStore store = org.eclipse.egit.ui.Activator.getDefault().getPreferenceStore();
 
